package cn.iocoder.yudao.module.customer.controller.admin.customerCompetitor;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerCompetitor.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCompetitor.CustomerCompetitorDO;
import cn.iocoder.yudao.module.customer.convert.customerCompetitor.CustomerCompetitorConvert;
import cn.iocoder.yudao.module.customer.service.customerCompetitor.CustomerCompetitorService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户竞争对手")
@RequestMapping("/customer/competitor")
public class CustomerCompetitorController {

    @Resource
    private CustomerCompetitorService competitorService;

    @PostMapping("/create")
    @ApiOperation("创建客户竞争对手")
    // @PreAuthorize("@ss.hasPermission('customer:competitor:create')")
    public CommonResult<CustomerCompetitorDO> createCompetitor(@Valid @RequestBody CustomerCompetitorCreateReqVO createReqVO) {
        return success(competitorService.createCompetitor(createReqVO));
    }

    // @PutMapping("/update")
    // @ApiOperation("更新客户竞争对手")
    // @PreAuthorize("@ss.hasPermission('customer:competitor:update')")
    // public CommonResult<Boolean> updateCompetitor(@Valid @RequestBody CustomerCompetitorUpdateReqVO updateReqVO) {
    //     competitorService.updateCompetitor(updateReqVO);
    //     return success(true);
    // }
    //
    // @DeleteMapping("/delete")
    // @ApiOperation("删除客户竞争对手")
    // @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:competitor:delete')")
    // public CommonResult<Boolean> deleteCompetitor(@RequestParam("id") Long id) {
    //     competitorService.deleteCompetitor(id);
    //     return success(true);
    // }
    //

     @GetMapping("/get")
     @ApiOperation("获得客户竞争对手")
     @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
     public CommonResult<CustomerCompetitorBackVO> getCompetitor(@RequestParam("id") Long id) {
         CustomerCompetitorDO competitor = competitorService.getCompetitor(id);
         return success(CustomerCompetitorConvert.INSTANCE.convert(competitor));
     }

    @GetMapping("/list")
    @ApiOperation("获得客户竞争对手列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    public CommonResult<List<CustomerCompetitorBackVO>> getCompetitorList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerCompetitorDO> list = competitorService.getCompetitorList(ids);
        return success(CustomerCompetitorConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/listAll")
    @ApiOperation("获得所有客户竞争对手列表")
    @ApiImplicitParam(name = "searchKey", value = "关键字", required = false, dataTypeClass = String.class)
    // @PreAuthorize("@ss.hasPermission('customer:competitor:query')")
    public CommonResult<List<CustomerCompetitorBackVO>> getCompetitorListAll(@RequestParam(value = "searchKey", required = false) String searchKey) {
        List<CustomerCompetitorDO> list = competitorService.getCompetitorListBySearchKey(searchKey);
        return success(CustomerCompetitorConvert.INSTANCE.convertList(list));
    }

    // @GetMapping("/page")
    // @ApiOperation("获得客户竞争对手分页")
    // @PreAuthorize("@ss.hasPermission('customer:competitor:query')")
    // public CommonResult<PageResult<CustomerCompetitorBackVO>> getCompetitorPage(@Valid CustomerCompetitorQueryVO query, PageVO page) {
    //     PageResult<CustomerCompetitorDO> pageResult = competitorService.getCompetitorPage(query, page);
    //     return success(CustomerCompetitorConvert.INSTANCE.convertPage(pageResult));
    // }

    // @GetMapping("/export-excel")
    // @ApiOperation("导出客户竞争对手 Excel")
    // @PreAuthorize("@ss.hasPermission('customer:competitor:export')")
    // @OperateLog(type = EXPORT)
    // public void exportCompetitorExcel(@Valid CustomerCompetitorQueryVO query,
    //           HttpServletResponse response) throws IOException {
    //     List<CustomerCompetitorDO> list = competitorService.getCompetitorList(query);
    //     // 导出 Excel
    //     List<CustomerCompetitorBackVO> datas = CustomerCompetitorConvert.INSTANCE.convertList(list);
    //     ExcelUtils.write(response, "客户竞争对手.xls", "数据", CustomerCompetitorBackVO.class, datas);
    // }
}
